# -*- coding: utf-8 -*-
'''
Created on Nov 30, 2015

@author: jakob
'''
from django.dispatch.dispatcher import Signal

add_to_index = Signal(providing_args=['instance', 'object_action'])
remove_from_index = Signal(providing_args=['instance', 'object_action'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.models import AnonymousUser
from django.template import RequestContext, Engine
from django.test import RequestFactory
from django.utils.text import smart_split

try:
    from django.utils.encoding import force_unicode
except ImportError:
    from django.utils.encoding import force_text as force_unicode

from cms.toolbar.toolbar import CMSToolbar

from .conf import settings
from .utils import get_field_value, strip_tags

EXCLUDED_PLUGINS = getattr(settings, 'ALDRYN_SEARCH_EXCLUDED_PLUGINS', [])


def _render_plugin(plugin, context, renderer=None):
    if renderer:
        content = renderer.render_plugin(
            instance=plugin,
            context=context,
            editable=False,
        )
    else:
        content = plugin.render_plugin(context)
    return content


def get_cleaned_bits(data):
    decoded = force_unicode(data)
    stripped = strip_tags(decoded)
    return smart_split(stripped)


def get_plugin_index_data(base_plugin, request):
    text_bits = []

    instance, plugin_type = base_plugin.get_plugin_instance()

    if instance is None or instance.plugin_type in EXCLUDED_PLUGINS:
        # this is an empty plugin or excluded from search
        return text_bits

    search_fields = getattr(instance, 'search_fields', [])
    if hasattr(instance, 'search_fulltext'):
        # check if the plugin instance has search enabled
        search_contents = instance.search_fulltext
    elif hasattr(base_plugin, 'search_fulltext'):
        # now check in the base plugin instance (CMSPlugin)
        search_contents = base_plugin.search_fulltext
    elif hasattr(plugin_type, 'search_fulltext'):
        # last check in the plugin class (CMSPluginBase)
        search_contents = plugin_type.search_fulltext
    else:
        # disabled if there's search fields defined,
        # otherwise it's enabled.
        search_contents = not bool(search_fields)

    if search_contents:
        context = RequestContext(request)
        updates = {}
        engine = Engine.get_default()

        for processor in engine.template_context_processors:
            updates.update(processor(context.request))
        context.dicts[context._processors_index] = updates

        try:
            # django-cms>=3.5
            renderer = request.toolbar.content_renderer
        except AttributeError:
            # django-cms>=3.4
            renderer = context.get('cms_content_renderer')

        plugin_contents = _render_plugin(instance, context, renderer)

        if plugin_contents:
            text_bits = get_cleaned_bits(plugin_contents)
    else:
        values = (get_field_value(instance, field) for field in search_fields)

        for value in values:
            cleaned_bits = get_cleaned_bits(value or '')
            text_bits.extend(cleaned_bits)
    return text_bits


def get_request(language=None):
    """
    Returns a Request instance populated with cms specific attributes.
    """
    request_factory = RequestFactory(HTTP_HOST=settings.ALLOWED_HOSTS[0])
    request = request_factory.get("/")
    request.session = {}
    request.LANGUAGE_CODE = language or settings.LANGUAGE_CODE
    # Needed for plugin rendering.
    request.current_page = None
    request.user = AnonymousUser()
    request.toolbar = CMSToolbar(request)
    return request



# -*- coding: utf-8 -*-
'''
Created on Dec 1, 2015

@author: jakob
'''
import copy

from haystack.exceptions import NotHandled
from haystack.signals import (
    RealtimeSignalProcessor as BaseRealtimeSignalProcessor
)
from .signals import add_to_index, remove_from_index


class RealtimeSignalProcessor(BaseRealtimeSignalProcessor):

    def setup(self):
        super(RealtimeSignalProcessor, self).setup()
        add_to_index.connect(self.handle_save)
        remove_from_index.connect(self.handle_delete)

    def teardown(self):
        super(RealtimeSignalProcessor, self).teardown()
        add_to_index.disconnect(self.handle_save)
        remove_from_index.disconnect(self.handle_delete)

    def handle_save(self, sender, instance, **kwargs):
        kwargs = copy.copy(kwargs)

        # Exact copy of Haystack's handle_save()
        # except that we pass any kwargs to the update_object method
        using_backends = self.connection_router.for_write(instance=instance)

        for using in using_backends:
            kwargs['using'] = using

            try:
                index = self.connections[using].get_unified_index().get_index(sender)
                # TODO: This should be done by haystack.
                index.update_object(instance, **kwargs)
            except NotHandled:
                # TODO: Maybe log it or let the exception bubble?
                pass



# -*- coding: utf-8 -*-
from django.utils.translation import get_language_from_request
from django.views.generic import ListView
from django.views.generic.edit import FormMixin

from haystack.forms import ModelSearchForm
from haystack.query import SearchQuerySet

from aldryn_common.paginator import DiggPaginator

from .conf import settings
from .utils import alias_from_language, get_model_path


class AldrynSearchView(FormMixin, ListView):
    form_class = ModelSearchForm

    # A list of models to limit search by.
    # Only indexes registered to these models will be searched.
    models = None

    load_all = False

    paginate_by = settings.ALDRYN_SEARCH_PAGINATION
    paginator_class = DiggPaginator

    # SearchQueryset instance to use for querying
    search_queryset = None
    # SearchQueryset class to instantiate if no search_queryset instance is defined.
    search_queryset_class = SearchQuerySet

    template_name = 'aldryn_search/search_results.html'

    def get_form_kwargs(self):
        kwargs = super(AldrynSearchView, self).get_form_kwargs()
        kwargs['load_all'] = self.load_all
        kwargs['searchqueryset'] = self.get_search_queryset()

        data = self.request.GET

        if self.models:
            data = data.copy()
            data.setlist('models', (get_model_path(model) for model in self.models))
        kwargs['data'] = data
        return kwargs

    def get_query(self, form):
        """
        Returns the query provided by the user.

        Returns an empty string if the query is invalid.
        """
        if form.is_valid():
            return form.cleaned_data['q']
        return ''

    def get(self, request, *args, **kwargs):
        form_class = self.get_form_class()
        self.form = self.get_form(form_class)
        return super(AldrynSearchView, self).get(request, *args, **kwargs)

    def get_queryset(self):
        queryset = self.form.search()
        if not self.request.user.is_authenticated():
            queryset = queryset.exclude(login_required=True)
        # TODO: fix that url filter.
        # url__in=['', None] make the query exclude "" and "None".
        # isnull=True, using Raw and other filters not works too
        # queryset = queryset.exclude(url__in=['', None])
        return queryset

    def get_search_queryset(self):
        if self.search_queryset is None:
            language = get_language_from_request(self.request, check_path=True)
            connection_alias = alias_from_language(language)
            return self.search_queryset_class(using=connection_alias)
        return self.search_queryset

    def get_context_data(self, **kwargs):
        context = super(AldrynSearchView, self).get_context_data(**kwargs)
        context['query'] = self.get_query(self.form)
        context['form'] = self.form
        if self.object_list.query.backend.include_spelling:
            context['suggestion'] = self.form.get_suggestion()
        return context
